package com.fzm.portal.controller;

import cn.hutool.crypto.SecureUtil;
import com.fzm.common.annotation.Authentication;
import com.fzm.common.constant.SystemConstant;
import com.fzm.common.entity.Nft;
import com.fzm.common.entity.NftDto;
import com.fzm.common.entity.User;
import com.fzm.common.entity.vo.CollectionNftVo;
import com.fzm.common.entity.vo.NftCertificateVo;
import com.fzm.common.entity.vo.NftVo;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.CategoryService;
import com.fzm.common.service.NftService;
import com.fzm.common.service.UserService;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.OssUtil;
import com.fzm.common.utils.QRCodeUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author tangtuo
 * @date 2021/6/30 15:08
 */
@Slf4j
@RestController
@RequestMapping(value = "/nft")
@Api(tags = "nft管理")
public class NftController {

    @Resource
    private OssUtil ossUtil;

    @Resource
    private NftService nftService;

    @Resource
    private UserService userService;

    @Resource
    private CategoryService categoryService;

    @Resource
    private HttpServletRequest request;

    /**
     * 存证二维码跳转地址
     * todo 修改成动态前端路由
     */
    private static final String PATH = "https://chain.33.cn/document/60";


    @Authentication
    @PostMapping("/save")
    @ApiOperation(value = "nft基本信息保存(基本信息和加密上链两个步骤)")
    public ResponseModel<NftDto> save(@ApiParam(value = "类目id", required = true) Integer categoryId,
                                      @ApiParam(value = "名称", required = true) String name,
                                      @ApiParam(value = "作者", required = true) String author,
                                      @ApiParam(value = "主题", required = true) String theme,
                                      @ApiParam(value = "简介", required = true) String synopsis,
                                      @ApiParam(value = "文件", required = false) MultipartFile file,
                                      @ApiParam(value = "文件hash", required = true) String fileHash,
                                      @ApiParam(value = "平台存档  0-不存档  1-加密存档", required = true) Integer isArchives,
                                      @ApiParam(value = "授权阅读  0-不需要授权  1-需要授权", required = false) Integer isGrant) throws IOException {
        Nft nft = new Nft();
        // 当平台存档选择加密存档时,文件必传
        if (SystemConstant.BOOLEAN_DATA_TRUE.equals(isArchives)) {
            if (file == null || isGrant == null) {
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "当选择加密存档的时候,文件和授权阅读不能为空");
            }
            // 判断文件的hash是否被篡改过
            log.info("verify fileHash start");
            if (!SecureUtil.md5(file.getInputStream()).equals(fileHash)) {
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "文件hash和文件内容不匹配");
            }
            log.info("verify fileHash end");
            log.info("upload file start");
            String fileUrl = ossUtil.putEncryptObject(file);
            log.info("upload file end");
            nft.setFileUrl(fileUrl).setIsGrant(isGrant).setFileName(file.getOriginalFilename());
        }
        nft.setCategoryId(categoryId)
                .setUserId(JwtUtil.getUserIdFromToken(request.getHeader("Authorization")))
                .setName(name)
                .setAuthor(author)
                .setTheme(theme)
                .setSynopsis(synopsis)
                .setFileHash(fileHash)
                .setIsArchives(isArchives);
        return ResponseModel.success(nftService.saveNft(nft));
    }


    @Authentication
    @PostMapping("/publish")
    @ApiOperation("发行nft")
    public ResponseModel<Boolean> publish(@Validated @RequestBody NftDto nftDto) {
        Boolean result = nftService.publish(nftDto);
        return ResponseModel.success(result);
    }

    @GetMapping("/list")
    @ApiOperation(value = "获取nft列表")
    public ResponseModel list(@ApiParam(value = "当前页码") @RequestParam Integer pageNum,
                              @ApiParam(value = "每页记录数") @RequestParam Integer pageSize,
                              @ApiParam(value = "类目id,查询所有的时候传null") @RequestParam(required = false) Integer categoryId) {

        pageNum = (pageNum - 1) * pageSize;
        List<Nft> list = nftService.list(pageNum, pageSize, categoryId);
        return ResponseModel.success(list);
    }

    @GetMapping("get/{id}")
    @ApiOperation(value = "获取nft详情")
    public ResponseModel<NftVo> getById(@PathVariable Integer id) {
        Nft nft = nftService.getById(id);
        if (nft == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此nft的详情");
        }
        User user = userService.getById(nft.getUserId());
        if (user == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此nft的发布人的信息");
        }
        NftVo nftVo = new NftVo(nft, user);
        nftVo.setCategory(categoryService.getById(nft.getCategoryId()).getCategoryName());
        return ResponseModel.success(nftVo);
    }

    @Authentication
    @GetMapping("/list/current")
    @ApiOperation(value = "获取我的nft列表")
    public ResponseModel<Map<String, Object>> listCurrent(@ApiParam(value = "类目id,查询全部的时候传null") @RequestParam(required = false) Integer categoryId) {
        List<Nft> list = nftService.listCurrent(categoryId, JwtUtil.getUserIdFromToken(request.getHeader("Authorization")));
        List<CollectionNftVo> nftVoList = list.stream()
                .map(nft -> new CollectionNftVo(nft, categoryService.getCategoryById(nft.getCategoryId())))
                .collect(Collectors.toList());
        HashMap<String, Object> result = new HashMap<>(4);
        result.put("list", nftVoList);
        result.put("size", nftVoList.size());
        return ResponseModel.success(result);
    }

    @Authentication
    @GetMapping("/generateNftId")
    @ApiOperation(value = "生成nft编号")
    public ResponseModel<String> generateNftId(@ApiParam(value = "类目id", required = true) @RequestParam Integer categoryId) {
        String nftId = nftService.generateNftId(categoryId);
        return ResponseModel.success(nftId);
    }

    @Authentication
    @PostMapping("/file/md5")
    @ApiOperation(value = "获取文件的md5值")
    public ResponseModel<String> getFileMd5Value(MultipartFile file) throws IOException {
        String md5 = SecureUtil.md5(file.getInputStream());
        return ResponseModel.success(md5);
    }


    @Authentication
    @GetMapping("/certificate/{id}")
    @ApiOperation(value = "nft证书")
    public ResponseModel<NftCertificateVo> certificate(@PathVariable Integer id, @RequestHeader String Authorization) throws Exception {
        Nft nft = nftService.getById(id);
        if (nft == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此nft的详情");
        }
        int userId = JwtUtil.getUserIdFromToken(Authorization);
        if (!nft.getUserId().equals(userId)) {
            throw GlobalException.newException(ResultCode.FORBIDDEN, "您无权查看别人的nft证书");
        }
        User user = userService.getById(userId);
        String qrCode = QRCodeUtil.encode(PATH, null, false);
        NftCertificateVo vo = new NftCertificateVo(nft, user, qrCode);
        return ResponseModel.success(vo);
    }

    @Authentication
    @ApiOperation(value = "下载文件")
    @PostMapping("/download/{id}")
    public void download(@ApiParam(value = "nft主键") @PathVariable Integer id) {
        nftService.download(id);
    }
}
